from django.db.models import Q

from netbox.filtersets import NetBoxModelFilterSet

from netbox_storage.models import WindowsVolume


class WindowsVolumeFilter(NetBoxModelFilterSet):
    """Filter capabilities for WindowsVolume instances."""

    class Meta:
        model = WindowsVolume
        fields = [
            "drive_name",
            "fs",
            "drives",
            "description"
        ]

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
            Q(drive_name__icontains=value)
            | Q(fs__icontains=value)
        )
        return queryset.filter(qs_filter)



from django.urls import path

from netbox.views.generic import ObjectChangeLogView

from netbox_storage.models import Drive, Filesystem, LinuxVolume, WindowsVolume
from netbox_storage.views import (
    # Drive View
    DriveListView,
    DriveView,
    DriveEditView,
    DriveDeleteView,
    DriveBulkImportView,
    DriveBulkEditView,
    DriveBulkDeleteView,
    # Filesystem Views
    FilesystemListView,
    FilesystemView,
    FilesystemEditView,
    FilesystemDeleteView,
    FilesystemBulkImportView,
    FilesystemBulkEditView,
    FilesystemBulkDeleteView,
    # LinuxVolume
    LinuxVolumeListView,
    LinuxVolumeView,
    LinuxVolumeDeleteView,
    LinuxVolumeEditView,
    LinuxVolumeBulkImportView,
    LinuxVolumeBulkEditView,
    LinuxVolumeBulkDeleteView,
    # WindowsVolume
    WindowsVolumeListView,
    WindowsVolumeView,
    WindowsVolumeDeleteView,
    WindowsVolumeEditView,
    WindowsVolumeBulkImportView,
    WindowsVolumeBulkEditView,
    WindowsVolumeBulkDeleteView,
)

app_name = "netbox_storage"

urlpatterns = [
    #
    # Drive urls
    #
    path("drive/", DriveListView.as_view(), name="drive_list"),
    path("drive/add/", DriveEditView.as_view(), name="drive_add"),
    path("drive/import/", DriveBulkImportView.as_view(), name="drive_import"),
    path("drive/edit/", DriveBulkEditView.as_view(), name="drive_bulk_edit"),
    path("drive/delete/", DriveBulkDeleteView.as_view(), name="drive_bulk_delete"),
    path("drive/<int:pk>/", DriveView.as_view(), name="drive"),
    path("drive/<int:pk>/edit/", DriveEditView.as_view(), name="drive_edit"),
    path("drive/<int:pk>/delete/", DriveDeleteView.as_view(), name="drive_delete"),
    path(
        "drive/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="drive_changelog",
        kwargs={"model": Drive},
    ),
    #
    # Filesystem urls
    #
    path("filesystem/", FilesystemListView.as_view(), name="filesystem_list"),
    path("filesystem/add/", FilesystemEditView.as_view(), name="filesystem_add"),
    path("filesystem/import/", FilesystemBulkImportView.as_view(), name="filesystem_import"),
    path("filesystem/edit/", FilesystemBulkEditView.as_view(), name="filesystem_bulk_edit"),
    path("filesystem/delete/", FilesystemBulkDeleteView.as_view(), name="filesystem_bulk_delete"),
    path("filesystem/<int:pk>/", FilesystemView.as_view(), name="filesystem"),
    path("filesystem/<int:pk>/edit/", FilesystemEditView.as_view(), name="filesystem_edit"),
    path("filesystem/<int:pk>/delete/", FilesystemDeleteView.as_view(), name="filesystem_delete"),
    path(
        "filesystem/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="filesystem_changelog",
        kwargs={"model": Filesystem},
    ),
    #
    # LinuxVolume urls
    #
    path("linuxvolume/", LinuxVolumeListView.as_view(), name="linuxvolume_list"),
    path("linuxvolume/add/", LinuxVolumeEditView.as_view(), name="linuxvolume_add"),
    path("linuxvolume/import/", LinuxVolumeBulkImportView.as_view(), name="linuxvolume_import"),
    path("linuxvolume/edit/", LinuxVolumeBulkEditView.as_view(), name="linuxvolume_bulk_edit"),
    path("linuxvolume/delete/", LinuxVolumeBulkDeleteView.as_view(), name="linuxvolume_bulk_delete"),
    path("linuxvolume/<int:pk>/", LinuxVolumeView.as_view(), name="linuxvolume"),
    path("linuxvolume/<int:pk>/edit/", LinuxVolumeEditView.as_view(), name="linuxvolume_edit"),
    path("linuxvolume/<int:pk>/delete/", LinuxVolumeDeleteView.as_view(), name="linuxvolume_delete"),
    path(
        "linuxvolume/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="linuxvolume_changelog",
        kwargs={"model": LinuxVolume},
    ),
    #
    # LinuxVolume urls
    #
    path("windowsvolume/", WindowsVolumeListView.as_view(), name="windowsvolume_list"),
    path("windowsvolume/add/", WindowsVolumeEditView.as_view(), name="windowsvolume_add"),
    path("windowsvolume/import/", WindowsVolumeBulkImportView.as_view(), name="windowsvolume_import"),
    path("windowsvolume/edit/", WindowsVolumeBulkEditView.as_view(), name="windowsvolume_bulk_edit"),
    path("windowsvolume/delete/", WindowsVolumeBulkDeleteView.as_view(), name="windowsvolume_bulk_delete"),
    path("windowsvolume/<int:pk>/", WindowsVolumeView.as_view(), name="windowsvolume"),
    path("windowsvolume/<int:pk>/edit/", WindowsVolumeEditView.as_view(), name="windowsvolume_edit"),
    path("windowsvolume/<int:pk>/delete/", WindowsVolumeDeleteView.as_view(), name="windowsvolume_delete"),
    path(
        "windowsvolume/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="windowsvolume_changelog",
        kwargs={"model": WindowsVolume},
    ),
]



from django.db import migrations, models
import utilities.json


class Migration(migrations.Migration):
    initial = True

    operations = [
        migrations.CreateModel(
            name="Drive",
            fields=[
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder
                    ),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("size", models.PositiveIntegerField()),
                ('cluster',
                 models.ForeignKey(on_delete=models.deletion.PROTECT, related_name='drive',
                                   to='virtualization.cluster')),
                ('virtual_machine',
                 models.ForeignKey(blank=True, null=True, on_delete=models.deletion.CASCADE,
                                   related_name='drive', to='virtualization.virtualmachine')),
                ("identifier", models.CharField(max_length=255)),
                ("description", models.CharField(max_length=255)),
            ],
            options={
                "ordering": ("size", "id"),
            },
        ),
        migrations.CreateModel(
            name="Filesystem",
            fields=[
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder
                    ),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("filesystem", models.CharField(max_length=255)),
                ("description", models.CharField(max_length=255)),
            ],
            options={
                "ordering": ("filesystem", "id"),
            },
        ),
        migrations.CreateModel(
            name="LinuxVolume",
            fields=[
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder
                    ),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("vg_name", models.CharField(max_length=255)),
                ("lv_name", models.CharField(max_length=255)),
                ("path", models.CharField(max_length=255)),
                ('fs',
                 models.ForeignKey(blank=True, null=True, on_delete=models.deletion.CASCADE, related_name='fs_linux',
                                   to='netbox_storage.filesystem')),
                ("description", models.CharField(max_length=255)),
            ],
            options={
                "ordering": ("lv_name", "id"),
            },
        ),
        migrations.CreateModel(
            name="WindowsVolume",
            fields=[
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder
                    ),
                ),
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("drive_name", models.CharField(max_length=255)),
                ("fs",
                 models.ForeignKey(blank=True, null=True, on_delete=models.deletion.CASCADE, related_name='fs_windows',
                                   to='netbox_storage.filesystem')),
                (
                    "drives",
                    models.ManyToManyField(
                        blank=True, related_name="drives", to="netbox_storage.drive"
                    ),
                ),
                ("description", models.CharField(max_length=255)),
            ],
            options={
                "ordering": ("drive_name", "id"),
            },
        ),
    ]



from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework.routers import APIRootView

from netbox.api.viewsets import NetBoxModelViewSet

from netbox_storage.api.serializers import (
    DriveSerializer,
    FilesystemSerializer, LinuxVolumeSerializer, WindowsVolumeSerializer,
)
from netbox_storage.filters import DriveFilter, FilesystemFilter, LinuxVolumeFilter, WindowsVolumeFilter
from netbox_storage.models import Drive, Filesystem, LinuxVolume, WindowsVolume


class NetboxStorageRootView(APIRootView):
    """
    NetboxDNS API root view
    """

    def get_view_name(self):
        return "NetboxStorage"


class DriveViewSet(NetBoxModelViewSet):
    queryset = Drive.objects.all()
    serializer_class = DriveSerializer
    filterset_class = DriveFilter

    @action(detail=True, methods=["get"])
    def drive(self, request, pk=None):
        drives = Drive.objects.filter(drive__id=pk)
        serializer = DriveSerializer(drives, many=True, context={"request": request})
        return Response(serializer.data)


class FilesystemViewSet(NetBoxModelViewSet):
    queryset = Filesystem.objects.all()
    serializer_class = FilesystemSerializer
    filterset_class = FilesystemFilter

    @action(detail=True, methods=["get"])
    def filesystem(self, request, pk=None):
        filesystem = Filesystem.objects.filter(filesystem__id=pk)
        serializer = FilesystemSerializer(filesystem, many=True, context={"request": request})
        return Response(serializer.data)


class LinuxVolumeViewSet(NetBoxModelViewSet):
    queryset = LinuxVolume.objects.all()
    serializer_class = LinuxVolumeSerializer
    filterset_class = LinuxVolumeFilter

    @action(detail=True, methods=["get"])
    def linuxvolume(self, request, pk=None):
        linuxvolume = LinuxVolume.objects.filter(linuxvolume__id=pk)
        serializer = LinuxVolumeSerializer(linuxvolume, many=True, context={"request": request})
        return Response(serializer.data)


class WindowsVolumeViewSet(NetBoxModelViewSet):
    queryset = WindowsVolume.objects.all()
    serializer_class = WindowsVolumeSerializer
    filterset_class = WindowsVolumeFilter

    @action(detail=True, methods=["get"])
    def windowsvolume(self, request, pk=None):
        windowsvolume = WindowsVolume.objects.filter(windowsvolume__id=pk)
        serializer = WindowsVolumeSerializer(windowsvolume, many=True, context={"request": request})
        return Response(serializer.data)
